import pygame, sys, os
import data
from pygame.locals import *

#translate from "ingame" coords to "real" coords
def to_real_coords((x,y)):
    #ingame has 40 16px tiles in width. 0-39
    #ingame has 20 24px tiles in height. 0-10
    #real has 640 x 480 px
    #real counts from top. ingame counts from bottom
    return (x*16, 480-y*24)



class Building:
    def __init__(self, filename = None):
        self.floors = []
        self.walls = []
        self.name = "unnamed"
        self.start = (0, 0)
        self.end = (10, 10)
        if filename:
            self.load_data(filename)

    def save_data(self):
        lvlnum = 1
        while os.path.exists("lvl%d" % lvlnum):
            lvlnum += 1
        filename = "lvl%d" % lvlnum
        f = open(filename, "w")
        f.write("name:%s\n" % self.name)
        f.write("start:%d %d\n" % self.start)
        f.write("end:%d %d\n" % self.end)

        for wall in self.walls:
            f.write("wall:%d %d %d\n" % (wall.x, wall.y, wall.height))

        for floor in self.floors:
            if floor.trampoline:
                f.write("trampoline:%d %d %d\n" % (floor.y, floor.startx, floor.stopx))
            else:
                f.write("floor:%d %d %d\n" % (floor.y, floor.startx, floor.stopx))
        f.close()

        
            
            
        

    def set_start(self, x, y):
        self.start = (x,y)

    def set_end(self, x, y):
        self.end = (x,y)

    def get_intersection_below(self, x, y):
        cur_y = 0
        intersect_floor = None
        for floor in self.floors:
            if floor.intersect(x) and floor.y <= y:
                if floor.y >= cur_y:
                    cur_y = floor.y
                    intersect_floor = floor
        #print cur_y
        return intersect_floor

    def get_intersection_above(self, x, y):
        cur_y = 1000
        intersect_floor = None
        for floor in self.floors:
            if floor.intersect(x) and floor.y > y:
                if floor.y <= cur_y:
                    cur_y = floor.y
                    intersect_floor = floor
        #print cur_y
        return intersect_floor

    def get_intersecting_wall(self, old_x, old_y, new_x, new_y, width):
        for wall in self.walls:
            if wall.intersect(old_x, old_y, new_x, new_y, width):
                return wall
        return None

    def get_wall_under(self, x, y):
        for wall in self.walls:
            if wall.is_under_object(x,y):
                return wall
        return None

    def get_wall_over(self, x, y):
        for wall in self.walls:
            if wall.is_over_object(x,y):
                return wall
        return None

    def does_sideway_collision(self, old_x, old_y, x, y):
        for floor in self.floors:
            if floor.does_sideway_collision_with(old_x, old_y, x, y):
                return True
        return False
                

    def make_floor(self, floor):
        self.floors.append(floor)

    def make_wall(self, wall):
        self.walls.append(wall)
                
    def draw_floors(self, surface):
        for floor in self.floors:
            floor.draw(surface)

    def draw_walls(self, surface):
        for wall in self.walls:
            wall.draw(surface)

    def draw(self, surface):
        self.draw_floors(surface)
        self.draw_walls(surface)
        self.draw_start_and_end(surface)

    def kill_at(self, x, y):
        for wall in self.walls:
            if wall.x == x:
                if wall.y <= y and (wall.y+wall.height) >= y:
                    self.walls.pop(self.walls.index(wall))
        for floor in self.floors:
            if floor.y == y:
                if floor.startx <= x and floor.stopx >= x:
                    self.floors.pop(self.floors.index(floor))
                

    def draw_start_and_end(self, surface):

        #start!
        pygame.draw.circle(surface, 0x00ff00, 
                           to_real_coords(self.start), \
                           5, 4)

        pygame.draw.circle(surface, 0xff0000, 
                           to_real_coords(self.end), \
                           5, 4)



        #end !

    def load_data(self, filename):
        f = data.load(filename)
        for line in f:
            
            if line.startswith("name"):
                self.name = line.split(":")
            else:
                args = map(int,line.split(":")[1].split(" "))
                if line.startswith("start"):
                    self.set_start(args[0], args[1])
                elif line.startswith("end"):
                    self.set_end(args[0], args[1])
                elif line.startswith("wall"):
                    self.make_wall(Wall(args[0], args[1], args[2]))
                elif line.startswith("floor"):
                    self.make_floor(Floor(args[0], args[1], args[2]))
                elif line.startswith("trampoline"):
                    self.make_floor(Trampoline(args[0], args[1], args[2]))
            
            

class Floor:
    def __init__(self, y, startx, stopx):

        self.trampoline = False
        self.y = y
        self.startx = startx
        self.stopx = stopx
        self.clr = 0

    def intersect(self, x):
        if ( x >= self.startx and x < self.stopx ) or ( (x+1) > self.startx and (x+1) <= self.stopx ) :
            return True
        else:
            return False

    def draw(self, surface):
        pygame.draw.line(surface, self.clr, \
                         to_real_coords((self.startx, self.y)), \
                         to_real_coords((self.stopx, self.y)), 3)

    def does_sideway_collision_with(self, old_x, old_y, x, y):
        if ((old_x > self.stopx) and (x < self.stopx)) or ((old_x < (self.startx-1)) and (x > self.startx -1)): # new intersection in x
            if ((self.y - y) < 1) and ((self.y - y) > 0):
                return True
        return False
        #DEPRACATED
        #        if (x > (self.startx-1)) and (x < self.stopx) and abs(y-self.y) < 0.9:
        #            return True
        #        return False
                
            
        

class Trampoline(Floor):
    def __init__(self, y, startx, stopx):
        Floor.__init__(self, y, startx, stopx)
        self.trampoline = True
        self.clr = 0xff0000


class Wall:
    def __init__(self, x, y, height):
        self.height = height
        self.x = x
        self.y = y

    def draw(self, surface):
        pygame.draw.line(surface, 0, \
                         to_real_coords((self.x, self.y)), \
                         to_real_coords((self.x, self.y+self.height)), 3)


    def intersect(self, old_x, old_y, new_x, new_y, width):
        if (new_y >= self.y and new_y < ( self.y + self.height)) or \
               ((new_y+0.7) > self.y and (new_y+1) <= ( self.y + self.height)) :
            #print old_x,":",self.x,":",new_x
            if ( (old_x+1) <= self.x and (new_x+1) >= self.x ):
                return True
            elif ( old_x >= self.x and new_x <= self.x ):
                return True
                
        return False


    def is_over_object(self, o_x, o_y):
        if ((self.x-o_x) < 1) and ((self.x-o_x) > 0) :
            if ( (self.y) <= o_y+1) and ((self.y) > o_y-0.5):
                return True



    def is_under_object(self, o_x, o_y):
        if ((self.x-o_x) < 1) and ((self.x-o_x) > 0) :
            if ( (self.y+self.height) > (o_y+0.1)) and ((self.y+self.height) < o_y+0.5):
                return True
        


class Grid:
    def __init__(self):
        pass

    def draw(self, surface):
        for x in range(40):
            for y in range(20):
                pygame.draw.circle(surface, 0, 
                                   to_real_coords((x, y)), \
                                   2, 1) 
class GameObject(pygame.sprite.Sprite):

    MAXSPEED = 0.2
    
    
    def __init__(self, x, y, parent, sprts, surface):

        self.surface = surface

        #set up sprite
        pygame.sprite.Sprite.__init__(self)
        self.images = []
        for sprt in sprts:
            img, rect = data.load_image(sprt)
            self.images.append(img)
        self.rect = rect
        #self.image, self.rect = data.load_image(sprt)

        self.x = x
        self.y = y
        self.accel_x = 0
        self.accel_y = -0.01
        self.accel_run = 0
        self.v_x = 0
        self.v_y = 0
        self.parent = parent
        self.wallhug_left = 0
        self.wallhug_right = 0

        self.switchcount = 6 # switch 10 times / sec
        self.curimg = 0
        self.image = self.images[self.curimg]

    def is_in_air(self):
        floor = self.parent.get_intersection_below(self.x, self.y)
        if floor:
            if floor.y < self.y:
                return True
        return False

    def update(self):
        
        #sprite handling
        #OPTIMIZE THIS!!!!!!
        if self.is_in_air():
            if self.wallhug_right or self.wallhug_left:
                self.curimg = 6
            #if self.curimg != 1:
            #    self.curimg = 1
            #    self.image = self.images[self.curimg]
        else:
            if self.v_x:
                self.switchcount -= 1
                if self.switchcount == 0:
                    self.switchcount = 6
                    self.curimg += 1
                    if self.curimg >= 8:
                        self.curimg = 0
            else:
                if self.curimg != 8:
                    self.curimg = 8


        if (self.v_x < 0) or self.wallhug_left:
            self.image = pygame.transform.flip(self.images[self.curimg], 1, 0)
        else:
            self.image = self.images[self.curimg]
        #END OF OPTIMIZE NEEDED

        
        old_x, old_y = self.x, self.y

        
        self.v_x += self.accel_x #+ self.accel_run

        #check for intersections with roof
        floor_above = self.parent.get_intersection_above(self.x, self.y)

        #check for intersections with floor
        floor = self.parent.get_intersection_below(self.x, self.y)
        if floor:
            if floor.y < self.y:
                #above a floor

                #update fallspeed
                #if wallhugging, then no acceleration!
                if not ( (self.wallhug_left) or (self.wallhug_right) ):
                    self.v_y += self.accel_y 
                else:
                    if self.v_y > 0:
                        self.v_y += self.accel_y*2
                    else:
                        self.v_y += self.accel_y/3 


                if self.v_x < GameObject.MAXSPEED and \
                    self.v_x > -GameObject.MAXSPEED:
                    self.v_x += self.accel_run/4
                    if self.wallhug_left and self.v_x < 0:
                        self.v_x = 0
                    if self.wallhug_right and self.v_x > 0:
                        self.v_x = 0
                
                #update y position
                self.y += self.v_y

                #dont fall through floor!
                if self.y < floor.y:
                    self.y = floor.y
                    self.v_y = 0
                    
            else:
                #at a floor

                #at a trampoline floor
                if floor.trampoline:
                    self.v_y += 0.3
                else:
                    pass
                
                if self.v_x < GameObject.MAXSPEED and \
                       self.v_x > -GameObject.MAXSPEED:
                    self.v_x += self.accel_run
                

                #Friction
                if not self.accel_run or self.accel_run*self.v_x < 0:
                    tresh = 0.03  #treshhold
                    if self.v_x > tresh:
                        self.v_x -= tresh
                    elif self.v_x < -tresh:
                        self.v_x += tresh
                    else:
                        self.v_x = 0
                
                #jump
                if self.v_y < 0:
                    self.v_y = 0
                elif self.v_y > 0:
                    self.y += self.v_y
                    

                
        else:
            #no floor!

            #update fallspeed
            #if wallhugging, then no acceleration!
            if not ( (self.wallhug_left) or (self.wallhug_right) ):
                self.v_y += self.accel_y 
            else:
                if self.v_y > 0:
                    self.v_y += self.accel_y*2
                else:
                    self.v_y += self.accel_y/3 


            #update position
            self.y += self.v_y


        #make sure you dont fall through the roof! :)
        if floor_above:
            if ((old_y + 0.9) < floor_above.y) and ((self.y+0.9) > floor_above.y):
                self.y = floor_above.y - 0.9
                self.v_y = 0


        #move!
        if (self.wallhug_left and self.v_x < 0 ) or (self.wallhug_right and self.v_x > 0):
            pass
        else:
            self.x += self.v_x
        

        #check wall intersection
        wall = self.parent.get_intersecting_wall(\
            old_x, old_y, self.x, self.y, self.rect.width)

        if wall:
            #print "INTERSECTING WALL!"
            #print "w_x",wall.x
            #print "old_x",old_x
            #print "new_x",self.x

            #self.v_y = 0
            if (old_x >= wall.x) and (self.x <= wall.x) :
                self.wallhug_right = True
                self.x = wall.x+0
            elif ((old_x+1) <= wall.x) and ((self.x+1) >= wall.x):
                self.wallhug_left = True
                self.x = wall.x-1

            else:
                "no hug"
            self.accel_x = 0
            self.v_x = 0

        #check wall intersection along Y-axis
        wall = self.parent.get_wall_under(self.x, self.y)
        if wall:
            #wall under player. stop movement!
            if self.is_in_air():
                self.y = old_y
                self.v_y = 0


        wall = self.parent.get_wall_over(self.x, self.y)

        if wall:
            #wall over player. stop movement!
            if self.v_y > 0:
                self.v_y = 0


        #check for sideways collision with floors
        if self.parent.does_sideway_collision(old_x, old_y, self.x, self.y):
            self.v_x = 0

        #sprite stuff
        ##move to new pos
        self.rect.bottomleft = to_real_coords((self.x, self.y))#to_real_coords((20,10))


class Hero(GameObject):
    def __init__(self, x, y, parent, sfc):
        GameObject.__init__(self, x, y, parent, ["walk%d.png"% n for n in range(1,9)]+["stand.png"], sfc)



    def moveStop(self):
        self.accel_run = 0

    def moveDown(self):
        print self.accel_x
        print self.v_x
        print self.accel_run
    def moveUp(self):
        if self.wallhug_left:
           self.wallhug_left = False
        if self.wallhug_right:
           self.wallhug_right = False
        if not self.is_in_air():
            self.v_y += 0.18
    def moveLeft(self):
        if self.wallhug_left:
            self.wallhug_left = False
            print "no more hugs from left!"
        self.accel_run = -0.008
    def moveRight(self):
        if self.wallhug_right:
            self.wallhug_right = False
            print "no more hugs from right!"
        self.accel_run = 0.008

    def dash(self):
        if self.is_in_air() and ( self.wallhug_right or self.wallhug_left ):
            self.wall_dash()
        elif not self.is_in_air():
            self.run_dash()
        

    def wall_dash(self):
        if self.wallhug_right:
            self.v_x += 0.2
            self.v_y += 0.2
            self.wallhug_right = False
            
        elif self.wallhug_left:
            self.v_x -= 0.2
            self.v_y += 0.2
            self.wallhug_left = False


    def run_dash(self):
        if self.v_x > 0:
            self.v_x += 0.1
            self.v_y += 0.1
        elif self.v_x < 0:
            self.v_x -= 0.1
            self.v_y += 0.1

def main():
    #pygames stuff
    pygame.init()
    SIZE = SCREENW, SCREENH = 640, 480
    screen = pygame.display.set_mode(SIZE)
    pygame.display.set_caption('Dashman DX')
    pygame.mouse.set_visible(1)
    background = data.load_image("bg.jpg")[0].convert()      
    clock = pygame.time.Clock()

    #my stuff
    building = Building("lvl9")


    #mf(1,1,30)
    #mt(2,5,40)
    #mf(5,5,15)
    #mf(7,7,28)
    #mf(3,7,16)
    
    #mw(1,1,10)
    #mw(30,1,10)

    #mw(15, 5, 12)
    #mw(18, 8, 7)


    grid = Grid()
    
    #building.make_floor(Floor(15,2,30))
    #building.make_floor(Floor(4,10,30))
    #building.make_floor(Floor(9,1,40))
    #building.make_floor(Floor(7,16,30))

    #building.make_wall(Wall(32,9,10))
    #building.make_wall(Wall(31,5,20))
    #building.make_wall(Wall(2,2,3))
    #building.make_wall(Wall(3,3,3))

    herox, heroy = building.start
    hero = Hero(herox, heroy, building, screen)

    allsprites = pygame.sprite.RenderPlain((hero,))

    to_make = 0

    last_point = (-1,-1)
    
    while(1):
        clock.tick(60)
        #hero.update()
        allsprites.update()
        #key events
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                sys.exit(1)
            elif event.type == pygame.KEYDOWN:
		if event.key == pygame.K_ESCAPE: sys.exit(0)
		elif event.key == pygame.K_SPACE:
                    hero.dash()
                
		elif event.key == pygame.K_UP:
                    hero.moveUp()
		elif event.key == pygame.K_LEFT:
                    hero.moveLeft()
		elif event.key == pygame.K_RIGHT:
                    hero.moveRight()
                elif event.key == pygame.K_s:
                    building.save_data()

                elif event.key == pygame.K_0: #none
                    to_make = 1
                elif event.key == pygame.K_1: #start
                    to_make = 1
                elif event.key == pygame.K_2: #end
                    to_make = 2
                elif event.key == pygame.K_3: #floor / wall
                    to_make = 3
                elif event.key == pygame.K_4: #wall
                    to_make = 4
                elif event.key == pygame.K_5: #trampoline
                    to_make = 5
                elif event.key == pygame.K_6:
                    to_make = 6
                elif event.key == pygame.K_7:
                    to_make = 7
                    

            elif event.type == pygame.KEYUP:
                if event.key == pygame.K_DOWN: pass
		elif event.key == pygame.K_UP:
                    pass
		elif event.key == pygame.K_LEFT:
                    hero.moveStop()
		elif event.key == pygame.K_RIGHT:
                    hero.moveStop()
            
            elif event.type == MOUSEBUTTONDOWN:

                (lb, mb, rb)= pygame.mouse.get_pressed()
                x = (pygame.mouse.get_pos()[0]+8)/16
                y = 20-(pygame.mouse.get_pos()[1]+12)/24

                #print pygame.mouse.get_pos()
                print x,y


                if rb:
                    building.kill_at(x,y)

                if lb:
                    if to_make == 1: #start
                        print "setting start!",x,"x",y
                        building.set_start(x,y)
                    elif to_make == 2: #end
                        print "setting end!",x,"x",y
                        building.set_end(x,y)
                    elif to_make == 3: #floor
                        #check if any last point
                        if last_point == (-1,-1): # no last point
                            last_point = (x,y)
                        else: #last point! draw some stuff!
                            #Draw a wall!
                            if (last_point[0] == x ) and (last_point[1] != y):
                                if last_point[1] < y:
                                    w_y = last_point[1]
                                    height = y - last_point[1]
                                else:
                                    w_y = y
                                    height = last_point[1] -y
                                building.make_wall(Wall(x, w_y, height))

                            #Draw a floor!
                            elif (last_point[1] == y) and (last_point[0] != x):
                                if last_point[0] < x:
                                    x1 = last_point[0]
                                    x2 = x
                                else:
                                    x1 = x
                                    x2 = last_point[0]
                                building.make_floor(Floor(y, x1, x2))
                            last_point = (-1, -1)
                    elif to_make == 4: #trampoline
                        if last_point == (-1,-1): # no last point
                            last_point = (x,y)
                        else: #last point! draw some stuff!
                            #Draw a trampoline floor!
                            if (last_point[1] == y) and (last_point[0] != x):
                                if last_point[0] < x:
                                    x1 = last_point[0]
                                    x2 = x
                                else:
                                    x1 = x
                                    x2 = last_point[0]
                                building.make_floor(Trampoline(y, x1, x2))
                            last_point = (-1, -1)
                        


                
                    

                
        screen.blit(background, (0, 0))

        #grid.draw(screen)
        building.draw(screen)
        allsprites.draw(screen)

        pygame.display.flip()
